# Code derived in part from
# https://stackoverflow.com/questions/19121446/python-object-history/19124089#19124089

import datetime


class History():
    def __init__(self, *args):
        pass

    def start(self, obj):
        if '_history' not in obj.__dict__:
            obj.__dict__['_history'] = {}

    def __call__(self, cls):
        this = self

        def getter(self, attr):
            this.start(self)
            if attr == 'history':
                return self._history
            if attr == 'historyTrace':
                return '\n'.join('%s: "%s" has changed to "%s"' % (t[0], t[1], t[2]) for t in self._history)
            return self.__dict__.get(attr)
        cls.__getattr__ = getter

        def setter(self, attr, value):
            this.start(self)
            if self._history.get(attr, False):
                if self._history[attr][-1] == value:
                    pass
                else:
                    self._history[attr].append(value)
            else:
                self._history[attr] = [value]
            self.__dict__[attr] = value
            # self._history[attr] = self._history.get(attr, []) + [(datetime.datetime.now(), attr, value)]
        cls.__setattr__ = setter

        return cls



#!/usr/bin/env python3
from glob import glob
from os.path import basename, splitext

from setuptools import find_packages, setup

setup(name='history-object',
      version="0.1.4",
      description='A python decorator that tracks added and changed elements of the decorated object.',
      packages=find_packages('lib'),
      package_dir={'': 'lib'},
      author='Connor Riva, Dave Lundgren',
      author_email='criva@westmont.edu, dlungren@outsideopen.com',
      url='https://bitbucket.org/westmont/history_object',
      py_modules=[splitext(basename(path))[0] for path in glob('lib/*.py')],
      keywords=['history', 'attributes', 'tracking', 'object', 'decorator'],
      classifiers=[
          'Development Status :: 3 - Alpha',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: MIT License',
          'Programming Language :: Python :: 2',
          'Programming Language :: Python :: 3',
      ],
      setup_requires=['setuptools-markdown'],
      long_description_markdown_filename='README.md'
      )
